/**
 * This class contains functions specific to the Load tab.
 * Its init() function should be called immediately upon successful project load.
 * @param {GenericTab} generic An optional GenericTab to prevent having to create multiple copies.
 */
function LoadTab(generic) {
    // PRIVATE MEMBERS \\       var private = "test";
    var gen = generic ? generic : new GenericTab();
    var id = "loadtab";
    var title = "Load";
    var deleteButtonId = "deletebutton";
    var deleteAllButtonId = "deleteAllButton";
    var importButtonId = "importButton";
    var uploadInputId = "uploadInput";
    var loadButtonId = "loadbutton";
    var dragAndDropDetectorId = "dragAndDropDetector";
    var dragAndDropOverlayId = "dragAndDropOverlay";

    // The currently selected slot's id.
    var selected = "";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * The onclick function for this tab.
     * Does nothing if the tab is already active.
     */
    function onClick() {
        if (gen.isActive(id)) {
            return;
        }
        gen.tabButtonTransition(id);
        global.sceneLoader.loadTabScene(id);
    }

    /**
     * Registers the onclick function for this tab.
     */
    function registerListeners() {
        $("#" + id).click(onClick);
    }

    /**
     * Builds the drag and drop overlay.
     * @returns {string} The drag and drop overlay.
     */
    function buildDragAndDropOverlay() {
        return `<div id="${dragAndDropOverlayId}" class="fullscreenpopup savedraganddropoverlay" style="display: none;"></div>`;
    }

    /**
     * Builds and returns the new game save slot HTML string.
     * @returns {string} The new game save slot HTML string.
     */
    function buildNewGameSlot() {
        var save = global.saveManager.getNewGameSaveMeta();
        return buildSmallSlotWithTime(save, "New game", false);
    }

    /**
     * Builds and returns the auto save slot HTML string.
     * @returns {string} The auto save slot HTML string.
     */
    function buildAutoSaveSlot() {
        var save = global.saveManager.getAutoSaveMeta();
        return buildSmallSlotWithTime(save, "Autosave", true);
    }

    /**
     * Builds and returns a small save slot, with comment box omitted.
     * @param {object} save The save to create a small slot for. 
     * @param {int} rank The rank (= slot number) of the save.
     * @param {boolean} displayTime If true, adds the time to the small save slot.
     * @returns {string} The save slot.
     */
    function buildSmallSlotWithTime(save, rank, displayTime) {
        var time = new Date(save.time);
        var slot = `<div class="${gen.smallSaveSlotClass}" id="${save.slot}">
                        <img class="savegender" src="${GenderEnum.properties[save.gender].icon}" alt="${GenderEnum.properties[save.gender].name}">
                        <div class="saverank">${rank}</div>`;
        if (displayTime) {
            slot += `   <div class="savetimestamp">${gen.timeToTimePassed(time)}</div>`;
        }

        slot += `    <div class="savename"><span class="bold">${save.name}</span></div>
                        <div class="savescene">${save.sceneName}</div>
                    </div>`;
        return slot;
    }

    /**
     * Builds and returns a generic save slot for loading purposes.
     * @param {object} save The save to create a slot for.
     * @returns {string} The save slot.
     */
    function buildSaveSlot(save) {
        var time = new Date(save.time);
        var slot = `<div class="${gen.saveSlotClass}" id="${save.slot}">
                        <img class="savegender" src="${GenderEnum.properties[save.gender].icon}" alt="${GenderEnum.properties[save.gender].name}">
                        <div class="saverank">${save.slot + 1}</div>
                        <div class="savetimestamp">${gen.timeToTimePassed(time)}</div>
                        <div class="savename"><span class="bold">${save.name}</span></div>
                        <div class="savescene">${save.sceneName}</div>
                        <textarea rows="2" cols="55" maxlength="114" class="savecomment" disabled>${save.comment}</textarea>
                    </div>`;
        return slot;
    }

    /**
     * Builds and returns the control bar, containing the load and delete buttons.
     */
    function buildControlBar() {
        var disableImport = global.saveManager.getFirstOpenSaveSlot() < 0;
        return `<div class="savecontrolbar">
                    <button class="button saveleftbutton warning" id="${deleteButtonId}" disabled>Delete</button>
                    <button class="button savesmallleftbutton warning" id="${deleteAllButtonId}">Delete ALL</button>
                    <button class="button savesmallrightbutton" id="${importButtonId}" ${disableImport ? "disabled" : ""}
                        title="You can also drag and drop save files onto the list above.">Import</button>
                    <button class="button saverightbutton" id="${loadButtonId}" disabled>Load</button>
                    <input id="${uploadInputId}" type="file" style="visibility:hidden;" accept=".kas" multiple>
                </div>`;
    }

    /**
     * Builds and returns an additional finisher.
     */
    function extraFinisher() {
        return `<div class="finish"></div>`;
    }

    /**
     * Builds and returns the save list.
     * @param {array} saveMeta The list of save metadata. 
     * @returns {string} The save list.
     */
    function buildSaveList(saveMeta) {
        var saveList = buildNewGameSlot();
        saveList += buildAutoSaveSlot();

        for (var i = 0; i < saveMeta.length; i++) {
            saveList += buildSaveSlot(saveMeta[i]);
        }

        return saveList;
    }

    /**
     * Disables the import button if there are no more free save slots.
     */
    function disableImportButtonIfRequired() {
        var disable = global.saveManager.getFirstOpenSaveSlot() < 0;
        if (disable) {
            gen.disable(importButtonId);
        }
    }

    /**
     * Builds the tab as a HTML string.
     * @returns {string} The tab, as a HTML string.
     */
    function buildTab() {
        var saveMeta = global.saveManager.saveList();
        var tab = gen.buildMaxSaveIndicator(saveMeta);
        tab += gen.buildTabHeader(title);
        var content = buildSaveList(saveMeta);
        content += extraFinisher();
        content += extraFinisher();
        content = `<div id="${dragAndDropDetectorId}" style="width: 100%; min-height: 80%;">` + content + `</div>`;
        tab += content;
        tab += gen.buildTabFooter();
        tab += buildDragAndDropOverlay();
        tab += buildControlBar();
        return tab;
    }



    /**
     * Registers listeners to the auto and new game slots.
     */
    function registerAutoAndNewListeners() {
        $("." + gen.smallSaveSlotClass).click(function () {
            gen.disable(deleteButtonId);
            gen.enable(loadButtonId);
            selectSlot($(this));
        });
    }

    /**
     * Function registered as click listeners to list elements (save slots).
     * "this" in this context refers to the save slot itself.
     * @returns {undefined}
     */
    function saveListClicked() {
        gen.enable(deleteButtonId);
        gen.enable(loadButtonId);
        selectSlot($(this));
    }

    /**
     * Registers listeners to the list of saves.
     */
    function registerListListeners() {
        $("." + gen.saveSlotClass).click(saveListClicked);
    }

    /**
     * Registers a listener for the delete button.
     */
    function registerDeleteButtonListener() {
        $("#" + deleteButtonId).click(function () {
            if (confirm("Are you sure you want to delete this save?")) {
                gen.disable(deleteButtonId);
                gen.disable(loadButtonId);
                gen.enable(importButtonId);
                global.saveManager.delete(selected);
                gen.updateMaxSaveIndicator();
                var toRemove = $("#" + selected);
                toRemove.hide(400, function () {
                    toRemove.remove();
                });
            }
        });
    }

    /**
     * Registers a listener for the load button.
     */
    function registerLoadButtonListener() {
        $("#" + loadButtonId).click(function () {
            gen.disable(deleteButtonId);
            gen.disable(loadButtonId);
            global.saveManager.load(selected);
        });
    }

    /**
     * Registers a listener for the delete all saves button.
     */
    function registerDeleteAllButtonListener() {
        $("#" + deleteAllButtonId).click(function () {
            if (!confirm("Do you really want to delete ALL your saves?")
                    || !confirm("There's no going back from this.")
                    || !confirm("Really consider the consequences of what you're doing.")
                    || !confirm("ALL your saves will be GONE. FOREVER.")
                    || confirm("Click CANCEL this time if you really want to go through with this.")
                    || !confirm("This is the last time I'm asking you.")
                    || !confirm("If you click OK, ALL your saves will be DELETED. Gone for good.")
                    ) {
                alert("Crisis averted.");
                return;
            }
            alert("So be it. I warned you.");
            gen.disable(deleteButtonId);
            gen.disable(loadButtonId);
            gen.enable(importButtonId);
            var meta = global.saveManager.saveList();
            for (var i = 0; i < meta.length; i++) {
                global.saveManager.delete(meta[i].slot);
            }
            var allElements = $("." + gen.saveSlotClass);
            allElements.hide(400, function () {
                allElements.remove();
            });
            gen.updateMaxSaveIndicator();
        });
    }

    /**
     * Registers a listener for the import button.
     */
    function registerImportButtonListener() {
        $("#" + importButtonId).click(function () {
            $("#" + uploadInputId).click();
        });
    }

    /**
     * Adds a save to the correct location in the list.
     * Animates the adding.
     * Assumes all previous save slots have been added.
     * @param {object} meta The metadata object of the slot to add.
     */
    function addSave(meta) {
        var slot = meta.slot;
        var previous = $("#" + (slot - 1));
        if (!saveExists(slot - 1)) {
            previous = $("#autosave");
        }
        var newSaveSlot = buildSaveSlot(meta);
        previous.after(newSaveSlot);
        var newSaveSlotElement = $("#" + slot);
        newSaveSlotElement.hide(0);
        newSaveSlotElement.show(400);
        newSaveSlotElement.click(saveListClicked);
    }

    /**
     * Checks whether or not the specified save exists in the load list.
     * @param {int} slot The slot of the save to check.
     * @returns {boolean} True if the slot exists, false otherwise.
     */
    function saveExists(slot) {
        var elem = $("#" + slot);
        return elem.length;
    }

    /**
     * Called when file import finishes.
     */
    function postImport() {
        $("#" + uploadInputId).val(""); // Reset to allow the same file to be uploaded twice in a row.
        var meta = global.saveManager.saveList();
        for (var i = 0; i < meta.length; i++) {
            var slot = meta[i].slot;
            if (!saveExists(slot)) {
                addSave(meta[i]);
            }
        }
        gen.enable(importButtonId);
        disableImportButtonIfRequired();
        gen.updateMaxSaveIndicator();
    }

    /**
     * Registers a change listener that will detect file imports.
     */
    function registerUploadChangeListener() {
        $("#" + uploadInputId).change(function (evt) {
            gen.disable(importButtonId);
            var list = evt.target.files;
            global.saveManager.import.files(list, postImport);
        });
    }

    var overlayCurrentlyDraggedOver = false;

    /**
     * Function called when a file gets hovered over the drag and drop area.
     * @param {event} evt The drag event.
     */
    function dropAreaDragOver(evt) {
        evt.preventDefault();
        evt.stopPropagation();
        if (overlayCurrentlyDraggedOver) {
            return;
        }
        overlayCurrentlyDraggedOver = true;
        var element = $("#" + dragAndDropOverlayId);
        element.stop(true);
        element.fadeIn(100);
    }

    /**
     * Function called when a file stops hovering over the drag and drop area.
     * @param {event} evt The drag leave event.
     */
    function dropAreaDragLeave(evt) {
        evt.preventDefault();
        evt.stopPropagation();
        if (!overlayCurrentlyDraggedOver) {
            return;
        }
        overlayCurrentlyDraggedOver = false;
        var element = $("#" + dragAndDropOverlayId);
        element.stop(true);
        element.fadeOut(100);
    }

    /**
     * Function called when a file gets dropped on the drag and drop area.
     * @param {event} evt The drop event.
     */
    function dropAreaDragDrop(evt) {
        evt.preventDefault();
        evt.stopPropagation();
        gen.disable(importButtonId);
        global.saveManager.import.files(evt.originalEvent.dataTransfer.files, postImport);
        overlayCurrentlyDraggedOver = false;
        var element = $("#" + dragAndDropOverlayId);
        element.stop(true);
        element.fadeOut(100);
    }

    /**
     * Registers a listener for the drop area.
     */
    function registerDropAreaListener() {
        $("#" + dragAndDropDetectorId).on("dragover", dropAreaDragOver);
        $("#" + dragAndDropOverlayId).on("dragover", dropAreaDragOver);
        $("#" + dragAndDropOverlayId).on("dragleave", dropAreaDragLeave);
        $("#" + dragAndDropOverlayId).on("drop", dropAreaDragDrop);
    }

    /**
     * Registers listeners to all load slots.
     */
    function registerLoadListeners() {
        registerAutoAndNewListeners();
        registerListListeners();
        registerDeleteButtonListener();
        registerDeleteAllButtonListener();
        registerLoadButtonListener();
        registerImportButtonListener();
        registerUploadChangeListener();
        registerDropAreaListener();
    }

    /**
     * Selects this element and unselects all others.
     * @param {DOMelement} element The element to select.
     */
    function selectSlot(element) {
        gen.removeSaveSlotSelected();
        element.addClass(gen.saveSlotSelectedClass);
        selected = element.attr('id');
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Initializes this tab.
     * Registers the tab click listener.
     * Registers the tab with the scene loader.
     */
    this.init = function () {
        registerListeners();
        global.sceneLoader.registerScene(id, this);
    };

    /**
     * Loads this tab.
     */
    this.load = function () {
        gen.load();
        var tab = buildTab();
        global.ui.loadToMain(tab);
        registerLoadListeners();
    };
}
