/**
 * Controller for the Introduction scene.
 * @param {string} sceneId The id of the scene.
 */
function IntroductionController(sceneId) {
    // PRIVATE MEMBERS \\       var private = "test";
    var id = sceneId;
    var genderChoice = "gender";
    var nameChoice = "name";
    var sanityLower = "sanity";
    var pressedContinue = "continue";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Sets the kobold's gender to the selected one.
     * @param {GenderEnum} choice The selected gender.
     */
    function genderChoiceGender(choice) {
        global.kobold.body.setGender(choice);
        if (choice == GenderEnum.MALE) {
            global.kobold.body.growDick(SizeEnum.SMALL);
        }
        if (choice == GenderEnum.FEMALE) {
            global.kobold.body.growBreasts(SizeEnum.SMALL);
        }
    }

    /**
     * Adds stats to the kobold depending on the chosen gender.
     * @param {GenderEnum} choice The selected gender.
     */
    function genderChoiceStats(choice) {
        if (choice == GenderEnum.MALE) {
            global.kobold.stats.increaseStat(StatsEnum.STRENGTH, 1);
            global.kobold.stats.increaseStat(StatsEnum.ENDURANCE, 1);
        }
        if (choice == GenderEnum.FEMALE) {
            global.kobold.stats.increaseStat(StatsEnum.AGILITY, 1);
            global.kobold.stats.increaseStat(StatsEnum.CHARISMA, 1);
        }
    }

    /**
     * Adds equipment to the kobold depending on the chosen gender.
     * @param {GenderEnum} choice The selected gender.
     */
    function genderChoiceEquipment(choice) {
        if (choice == GenderEnum.MALE) {
            global.kobold.inventory.equipment.wear(new LoinCloth());
        }
        if (choice == GenderEnum.FEMALE) {
            global.kobold.inventory.equipment.wear(new LinenPants());
            global.kobold.inventory.equipment.wear(new ChestWrap());
        }
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Makes the gender choice, setting the kobold's gender and altering the kobold's history.
     * @param {GenderEnum} choice The selected gender.
     */
    this.makeGenderChoice = function (choice) {
        var history = global.kobold.history.get(id);
        history[genderChoice] = choice;
        global.kobold.history.set(id, history);
        genderChoiceGender(choice);
        genderChoiceStats(choice);
        genderChoiceEquipment(choice);
    };

    /**
     * Fetches the gender choice from history and returns it.
     * May return undefined if the choice has not yet been made.
     */
    this.getGenderChoice = function () {
        var history = global.kobold.history.get(id);
        return history[genderChoice];
    };

    /**
     * Returns true if the gender choice has already been made. False otherwise.
     */
    this.hasMadeGenderChoice = function () {
        return global.kobold.history.has(id, genderChoice);
    };

    /**
     * Makes the name choice, setting the kobold's name.
     * @param {string} name The entered name.
     */
    this.makeNameChoice = function (name) {
        var history = global.kobold.history.get(id);
        history[nameChoice] = name;
        global.kobold.history.set(id, history);
        global.kobold.name = name;
    };

    /**
     * Fetches the name choice from history and returns it.
     * May return undefined if the choice has not yet been made.
     */
    this.getNameChoice = function () {
        var history = global.kobold.history.get(id);
        return history[nameChoice];
    };

    /**
     * Returns true if the name choice has already been made. False otherwise.
     */
    this.hasMadeNameChoice = function () {
        return global.kobold.history.has(id, nameChoice);
    };
    
    /**
     * Presses the continue button.
     */
    this.pressContinue = function () {
        var history = global.kobold.history.get(id);
        history[pressedContinue] = true;
        global.kobold.history.set(id, history);
    };
    
    /**
     * Returns true if the continue button has been pressed.
     */
    this.hasContinued = function () {
        return global.kobold.history.has(id, pressedContinue);
    };

    /**
     * Lowers the kobold's sanity, if it hasn't been lowered yet.
     * @returns {string} HTML describing the change.
     */
    this.thirdPartMentalEffects = function () {
        if (!global.kobold.history.has(id, sanityLower)) {
            global.kobold.status.mental.decrease(15, "Deadline worries");
            var history = global.kobold.history.get(id);
            history[sanityLower] = true;
            global.kobold.history.set(id, history);
        }
        return global.ui.change.buildNotification([new Change("Distraught", "Sanity", -15)]);
    };
}
